@@ -1,7 +1,7 @@
-/*	$Id: ldapvacation.c,v 1.47 2006/03/12 09:34:04 mbalmer Exp $	*/
+/*	$Id: ldapvacation.c,v 1.53 2006/06/24 09:52:15 mbalmer Exp $	*/
 
 /*
- * Copyright (c) 2005,2006 Marc Balmer <marc@msys.ch>
+ * Copyright (c) 2005, 2006 Marc Balmer <marc@msys.ch>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -37,16 +37,12 @@
 #include <string.h>
 #include <syslog.h>
 #include <time.h>
+#include <tzfile.h>
 #include <unistd.h>
 
 #include "ldapvacation.h"
 #include "pathnames.h"
 
-#ifdef __linux__
-#include "strlfunc.h"
-#include "strtonum.h"
-#endif
-
 /*
  * LDAPVACATION -- return a message to the sender when on vacation.
  *
@@ -64,23 +60,13 @@
 #define LDAPHOST	"localhost"
 #define LDAPPORT	389
 #define LDAPURL		NULL
-#define LDAPQUERY	"(&(objectClass=vmailAccount)(vacationEnabled=TRUE)(uid=$u))"
+#define LDAPQUERY	"(&(objectClass=vmailAccount)(vacationEnabled=TRUE)" \
+			    "(uid=$u))"
 #define MAILATTR	"mailLocalAddress"
 
-#if (__linux__)
-#define SECSPERMIN      60
-#define MINSPERHOUR     60
-#define HOURSPERDAY     24
-#define DAYSPERWEEK     7
-#define SECSPERHOUR     (SECSPERMIN * MINSPERHOUR)
-#define SECSPERDAY      ((long) SECSPERHOUR * HOURSPERDAY)
-#else
-#include <tzfile.h>
-#endif
-
 LDAP	*ld;
 char	*ldaphost;
-unsigned int	 ldapport;
+int	 ldapport;
 char	*ldapurl;
 char	*searchbase;
 char	*binddn;
@@ -216,7 +202,8 @@ readheaders(void)
 			goto findme;
 		case 'X':		/* X-Spam-Status: spam" */
 		case 'x':
-			if (spamheader != NULL && !strncasecmp(buf, spamheader, strlen(spamheader)))
+			if (spamheader != NULL &&
+			    !strncasecmp(buf, spamheader, strlen(spamheader)))
 				exit(0);
 			break;
 		default:
@@ -271,7 +258,7 @@ junkmail(void)
 		{ NULL, 0 }
 	};
 	struct ignore *cur;
-	int len;
+	size_t len;
 	char *p;
 
 	/*
@@ -319,38 +306,44 @@ recent(void)
 	int n;
 	size_t l;
 	
-	next = interval == -1 ? (time_t)SECSPERDAY * (time_t)DAYSPERWEEK : interval;
+	next = interval == -1 ? (time_t)SECSPERDAY * (time_t)DAYSPERWEEK :
+	    interval;
 	
 	if (*query == '(')
-		snprintf(recent_query, sizeof(recent_query), "(&%s(vacationSent=%s *))", query, from);
+		snprintf(recent_query, sizeof(recent_query),
+		    "(&%s(vacationSent=%s *))", query, from);
 	else
-		snprintf(recent_query, sizeof(recent_query), "(&(%s)(vacationSent=%s *))", query, from);
+		snprintf(recent_query, sizeof(recent_query),
+		    "(&(%s)(vacationSent=%s *))", query, from);
 
-	if (ldap_search_s(ld, searchbase, LDAP_SCOPE_SUBTREE, recent_query, attrs, 0, &result) == LDAP_SUCCESS) {
-		if (ldap_count_entries(ld, result)) {
-			e = ldap_first_entry(ld, result);
-			vals = ldap_get_values(ld, e, "vacationSent");
-			for (n = 0, l = strlen(from); vals[n] != NULL; n++) {
-				if (!strncasecmp(vals[n], from, l)) {
-					if ((vacationSent = strdup(vals[n])) == NULL) {
-						syslog(LOG_ERR, "memory allocation error");
-						exit(1);
-					}
-					if ((s = strchr(vals[n], ' '))) {
-						then = (time_t) atol(++s);
+	if (ldap_search_s(ld, searchbase, LDAP_SCOPE_SUBTREE, recent_query,
+	    attrs, 0, &result) != LDAP_SUCCESS)
+		return retval;
 
-						if (next == (time_t) LONG_MAX ||
-							then + next > time(NULL)) {
-							retval = 1;
-							break;
-						}
+	if (ldap_count_entries(ld, result)) {
+		e = ldap_first_entry(ld, result);
+		vals = ldap_get_values(ld, e, "vacationSent");
+		for (n = 0, l = strlen(from); vals[n] != NULL; n++) {
+			if (!strncasecmp(vals[n], from, l)) {
+				if ((vacationSent = strdup(vals[n])) == NULL) {
+					syslog(LOG_ERR, "memory "
+					    "allocation error");
+					exit(1);
+				}
+				if ((s = strchr(vals[n], ' '))) {
+					then = (time_t) atol(++s);
+
+					if (next == (time_t) LONG_MAX
+					    || then + next > time(NULL)) {
+						retval = 1;
+						break;
 					}
 				}
 			}
-			ldap_value_free(vals);
 		}
-		ldap_msgfree(result);
+		ldap_value_free(vals);
 	}
+	ldap_msgfree(result);
 	return retval;
 }
 
@@ -376,7 +369,8 @@ setreply(void)
 		vacationvals[0] = vacationSent;
 		mod.mod_op = LDAP_MOD_DELETE;
 		if (ldap_modify_s(ld, dn, mods) != LDAP_SUCCESS) {
-			syslog(LOG_ERR, "Can't delete old vacation sender for %s", dn);
+			syslog(LOG_ERR, "can't delete old vacation sender "
+			    "for %s", dn);
 			exit(1);
 		}
 	}
@@ -402,42 +396,12 @@ sendmessage(char *myname, volatile char *msg)
 	char cmd[MAXLINE];
 	char *s;
 
-#if 0
-	int pvect[2], i;
-
-	The original vacation program uses vfork() to start sendmail.
-	After problems using this from maildrop, I switched to popen().
-	
-	if (pipe(pvect) < 0) {
-		syslog(LOG_ERR, "pipe: %m");
-		exit(1);
-	}
-	i = vfork();
-	if (i < 0) {
-		syslog(LOG_ERR, "fork: %m");
-		exit(1);
-	}
-	if (i == 0) {
-		dup2(pvect[0], 0);
-		close(pvect[0]);
-		close(pvect[1]);
-		execl(_PATH_SENDMAIL, "sendmail", "-v", "-f", myname, "--",
-		    from, (char *)NULL);
-		syslog(LOG_ERR, "can't exec %s: %m", _PATH_SENDMAIL);
-		_exit(1);
-	}
-	close(pvect[0]);
-	sfp = fdopen(pvect[1], "w");
-	if (sfp == NULL) {
-		close(pvect[1]);
-		return;
-	}
-#endif
-
-	snprintf(cmd, sizeof(cmd), "%s -f %s -- %s", _PATH_SENDMAIL, myname, from);
+	snprintf(cmd, sizeof(cmd), "%s -f '%s' -- '%s'", _PATH_SENDMAIL, myname,
+	    from);
 
 	if (verbose)
-		syslog(LOG_INFO, "send vacation msg from %s to %s using command '%s'", myname, from, cmd);
+		syslog(LOG_INFO, "send vacation msg from %s to %s using "
+		    "command '%s'", myname, from, cmd);
 
 	sfp = popen(cmd, "w");
 	if (sfp == NULL) {
@@ -463,19 +427,13 @@ sendmessage(char *myname, volatile char *msg)
 __dead void
 usage(void)
 {
-	fprintf(stderr, "usage: %s [-tVvZ[Z]] [-b searchbase] [-C configfile] [-D binddn] "
-	    "[-d dest] [-f from] [-h ldaphost] [-L ldapport] "
-	    "[-m mailattr] [-q query] [-u ldapurl] [-w bindpasswd] [-x spamheader]\n", __progname);
+	fprintf(stderr, "usage: %s [-VvZ[Z]] [-b searchbase] [-C configfile] "
+	    "[-D binddn] [-d dest] [-f from] [-h ldaphost] [-L ldapport] "
+	    "[-m mailattr] [-q query] [-u ldapurl] [-w bindpasswd] "
+	    "[-x spamheader]\n", __progname);
 	exit(1);
 }
 
-void
-exithandler(void)
-{
-	closelog();
-	ldap_unbind(ld);
-}
-
 int
 main(int argc, char *argv[])
 {
@@ -520,8 +478,8 @@ main(int argc, char *argv[])
 	spamheader = NULL;
 	
 	/* Process the commandline */
-
-	while ((ch = getopt(argc, argv, "b:C:Dd:f:h:m:p:q:tu:Vvw:x:?")) != -1) {
+	while ((ch = getopt(argc, argv, "b:C:Dd:f:h:m:p:q:tu:Vvw:x:Z?")) !=
+	    -1) {
 		switch (ch) {
 		case 'b':
 			searchbase = optarg;
@@ -549,23 +507,19 @@ main(int argc, char *argv[])
 			mailattr = optarg;
 			break;
 		case 'p':
-			ldapport = (unsigned int)strtonum(optarg, 1, 65535, &errstr);
+			ldapport = (int)strtonum(optarg, 1LL, 65535LL, &errstr);
 			if (errstr)
 				errx(1, "port number is %s:%s", errstr, optarg);
 			break;
 		case 'q':
 			query = optarg;
 			break;
-		case 't':
-			ldap_use_tls = 1;
-			break;
 		case 'u':
 			ldapurl = optarg;
 			break;
 		case 'V':
 			printf("%s %s\n", __progname, VERSION);
 			exit(0);
-			/* FALLTHROUGH */
 		case 'v':
 			verbose = 1;
 			break;
@@ -576,7 +530,8 @@ main(int argc, char *argv[])
 			if (*optarg == 'x' || *optarg == 'X')
 				spamheader = optarg;
 			else {
-				fprintf(stderr, "Spam header must start with 'x' or 'X'");
+				fprintf(stderr, "Spam header must start with "
+				    "'x' or 'X'");
 				exit(1);
 			}
 			break;
@@ -590,12 +545,10 @@ main(int argc, char *argv[])
 				break;
 			default:
 				usage();
-				/* NOTREACHED */
 			}
 			break;
 		default:
 			usage();
-			/* NOTREACHED */
 		}
 	}
 
@@ -603,7 +556,6 @@ main(int argc, char *argv[])
 	argv += optind;
 
 	/* Read config file */
-
 	ldapvacation_init();
 
 	if (uid == NULL) {
@@ -617,7 +569,6 @@ main(int argc, char *argv[])
 	}
 
 	/* Set default values if some variables are not set */
-
 	if (ldaphost == NULL)
 		ldaphost = LDAPHOST;
 	if (ldapport == 0)
@@ -643,35 +594,41 @@ main(int argc, char *argv[])
 		exit(1);
 	}
 
-	openlog(__progname, LOG_CONS | LOG_NDELAY | LOG_PID | verbose ? LOG_PERROR : 0, log_facility);
+	openlog(__progname, LOG_CONS | LOG_NDELAY | LOG_PID | verbose ?
+	    LOG_PERROR : 0, log_facility);
 
 	if (ldapurl == NULL) {
 		if ((ld = ldap_init(ldaphost, ldapport)) == NULL) {
-			syslog(LOG_ERR, "No directory server at %s, %m", ldaphost);
+			syslog(LOG_ERR, "No directory server at %s, %m",
+			    ldaphost);
 			exit(1);
 		}
 	} else {
 		if (ldap_initialize(&ld, ldapurl)) {
-			syslog(LOG_ERR, "No directory server at %s, %m", ldapurl);
+			syslog(LOG_ERR, "No directory server at %s, %m",
+			    ldapurl);
 			exit(1);
 		}
 	}
 
 	version = LDAP_VERSION3;
-	if (ldap_set_option(ld, LDAP_OPT_PROTOCOL_VERSION, &version) != LDAP_OPT_SUCCESS) {
+	if (ldap_set_option(ld, LDAP_OPT_PROTOCOL_VERSION, &version) !=
+	    LDAP_OPT_SUCCESS) {
 		syslog(LOG_ERR, "Failed to set LDAP version 3 protocol");
 		exit(1);
 	}
 
-	if (ldap_use_tls > 0 && ldap_start_tls_s(ld, NULL, NULL) != LDAP_SUCCESS) {
+	if (ldap_use_tls > 0 && ldap_start_tls_s(ld, NULL, NULL) !=
+	    LDAP_SUCCESS) {
 		syslog(LOG_ERR, "Failed to start TLS for LDAP");
 		if (ldap_use_tls > 1)
 			exit(1);
 	}
 
 	if (ldap_simple_bind_s(ld, binddn, bindpasswd) != LDAP_SUCCESS) {
-		syslog(LOG_ERR, "Failed to bind to directory server as '%s'", binddn);
-		memset(bindpasswd, 0L, strlen(bindpasswd));
+		syslog(LOG_ERR, "Failed to bind to directory server as '%s'",
+		    binddn);
+		bzero(bindpasswd, strlen(bindpasswd));
 		exit(1);
 	}
 	bzero(bindpasswd, strlen(bindpasswd));
@@ -679,12 +636,6 @@ main(int argc, char *argv[])
 	if (verbose)
 		syslog(LOG_INFO, "bound to LDAP server as %s", binddn);
 
-	if (atexit(exithandler)) {
-		exithandler();
-		syslog(LOG_ERR, "Can't install exit handler");
-		exit(1);
-	}
-
 	if ((s = strstr(query, "$u")) != NULL) {
 		size = strlen(query) + strlen(uid) - 1;
 
@@ -705,7 +656,8 @@ main(int argc, char *argv[])
 	if (verbose)
 		syslog(LOG_INFO, "LDAP query: %s", query);
 
-	if (ldap_search_s(ld, searchbase, LDAP_SCOPE_SUBTREE, query, attrs, 0, &result) == LDAP_SUCCESS) {
+	if (ldap_search_s(ld, searchbase, LDAP_SCOPE_SUBTREE, query, attrs,
+	    0, &result) == LDAP_SUCCESS) {
 		if (!ldap_count_entries(ld, result))
 			exit(0);	
 
@@ -722,8 +674,9 @@ main(int argc, char *argv[])
 		if (verbose)
 			syslog(LOG_INFO, "DN: %s", dn);
 
-		if ((vals = ldap_get_values(ld, e, "vacationInterval")) != NULL) {
-			interval = (time_t) atol(vals[0]);
+		if ((vals = ldap_get_values(ld, e, "vacationInterval")) !=
+		    NULL) {
+			interval = (time_t)atol(vals[0]);
 			ldap_value_free(vals);
 		}
 
@@ -731,12 +684,13 @@ main(int argc, char *argv[])
 			for (; *vals != NULL; vals++) {
 				if (myname == NULL)
 					myname = *vals;
-				if ((cur = (ALIAS *) malloc(sizeof(ALIAS))) == NULL) {
+				if ((cur = malloc(sizeof(ALIAS))) == NULL) {
 					syslog(LOG_ERR, "memory error");
 					exit(1);
 				}
 				if (verbose)
-					syslog(LOG_INFO, "%s: %s", mailattr, *vals);
+					syslog(LOG_INFO, "%s: %s", mailattr,
+					    *vals);
 					
 				cur->name = *vals;
 				cur->next = names;
@@ -758,7 +712,8 @@ main(int argc, char *argv[])
 		if (!recent()) {
 			if (verbose)
 				syslog(LOG_INFO, "not recent");
-			if ((vals = ldap_get_values(ld, e, "vacationMsg")) != NULL) {
+			if ((vals = ldap_get_values(ld, e, "vacationMsg")) !=
+			    NULL) {
 				sendmessage(myname, vals[0]);
 				setreply();
 				ldap_value_free(vals);
